@@ -1,15 +1,21 @@
 /*
  *  Copyright (C) 1999-2002 Bernd Gehrmann
  *                          bernd@mail.berlios.de
- *  Copyright (c) 2002-2004 Christian Loose <christian.loose@kdemail.net>
+ *  Copyright (c) 2002-2005 Christian Loose <christian.loose@kdemail.net>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include <qlabel.h>
@@ -88,7 +94,7 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
     , opt_doCVSEdit( false )
     , recent( 0 )
     , cvsService( 0 )
-    , statusBar( 0 )
+    , m_statusBar(new KParts::StatusBarExtension(this))
     , m_browserExt( 0 )
     , filterLabel( 0 )
     , m_editWithId(0)
@@ -145,18 +151,6 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
                                   "cvs DCOP service could not be started."),
                              parentWidget));
 
-    statusBar = new CervisiaStatusBarExtension(this);
-
-    // create the active filter indicator and add it to the statusbar
-    filterLabel = new QLabel("UR", statusBar->statusBar());
-    filterLabel->setFixedSize(filterLabel->sizeHint());
-    filterLabel->setText("");
-    QToolTip::add(filterLabel,
-                  i18n("F - All files are hidden, the tree shows only folders\n"
-                       "N - All up-to-date files are hidden\n"
-                       "R - All removed files are hidden"));
-    statusBar->addStatusBarItem(filterLabel, 0, true);
-
     if( cvsService )
     {
         setupActions();
@@ -165,6 +159,8 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
     }
 
     setXMLFile( "cervisiaui.rc" );
+
+    QTimer::singleShot(0, this, SLOT(slotSetupStatusBar()));
 }
 
 CervisiaPart::~CervisiaPart()
@@ -207,6 +203,20 @@ bool CervisiaPart::openURL( const KURL &u )
     return openSandbox( u.path() );
 }
 
+
+void CervisiaPart::slotSetupStatusBar()
+{
+    // create the active filter indicator and add it to the statusbar
+    filterLabel = new QLabel("UR", m_statusBar->statusBar());
+    filterLabel->setFixedSize(filterLabel->sizeHint());
+    filterLabel->setText("");
+    QToolTip::add(filterLabel,
+                  i18n("F - All files are hidden, the tree shows only folders\n"
+                       "N - All up-to-date files are hidden\n"
+                       "R - All removed files are hidden"));
+    m_statusBar->addStatusBarItem(filterLabel, 0, true);
+}
+
 void CervisiaPart::setupActions()
 {
     KAction *action;
@@ -677,12 +687,14 @@ void CervisiaPart::aboutCervisia()
                           "Copyright (c) 1999-2002\n"
                           "Bernd Gehrmann <bernd@mail.berlios.de>\n"
                           "\n"
-                          "This program may be distributed under the terms of the Q Public\n"
-                          "License as defined by Trolltech AS of Norway and appearing in the\n"
-                          "file LICENSE.QPL included in the packaging of this file.\n\n"
+                          "This program is free software; you can redistribute it and/or modify\n"
+                          "it under the terms of the GNU General Public License as published by\n"
+                          "the Free Software Foundation; either version 2 of the License, or\n"
+                          "(at your option) any later version.\n"
                           "This program is distributed in the hope that it will be useful,\n"
                           "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-                          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"
+                          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+                          "GNU General Public License for more details.\n"
                           "See the ChangeLog file for a list of contributors."));
     QMessageBox::about(0, i18n("About Cervisia"),
                        aboutstr.arg(CERVISIA_VERSION).arg(KDE_VERSION_STRING));
@@ -694,7 +706,7 @@ KAboutData* CervisiaPart::createAboutData()
     KAboutData* about = new KAboutData(
                             "cervisiapart", I18N_NOOP("Cervisia Part"),
                             CERVISIA_VERSION, I18N_NOOP("A CVS frontend"),
-                            KAboutData::License_QPL,
+                            KAboutData::License_GPL,
                             I18N_NOOP("Copyright (c) 1999-2002 Bernd Gehrmann"), 0,
                             "http://www.kde.org/apps/cervisia");
 
@@ -1734,7 +1746,8 @@ void CervisiaPart::setFilter()
                 str += "R";
         }
 
-    filterLabel->setText(str);
+    if( filterLabel )
+        filterLabel->setText(str);
 }
 
 
