@@ -1,15 +1,21 @@
 /*
  *  Copyright (C) 1999-2002 Bernd Gehrmann
  *                          bernd@mail.berlios.de
- *  Copyright (c) 2002-2003 Christian Loose <christian.loose@hamburg.de>
+ *  Copyright (c) 2002-2004 Christian Loose <christian.loose@kdemail.net>
  *
- * This program may be distributed under the terms of the Q Public
- * License as defined by Trolltech AS of Norway and appearing in the
- * file LICENSE.QPL included in the packaging of this file.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 
@@ -20,6 +26,7 @@
 
 class KRecentFilesAction;
 
+
 /**
  * A basic shell that embeds the Cervisia part directly to make a standalone
  * GUI available.
@@ -29,11 +36,9 @@ class CervisiaShell : public KParts::MainWindow
     Q_OBJECT
 
 public:
-    CervisiaShell( const char *name=0 );
+    CervisiaShell(const char* name=0);
     virtual ~CervisiaShell();
 
-    void restorePseudo(const QString &dirname);
-
 public slots:
     void openURL();
     void openURL(const KURL& url);
@@ -47,15 +52,18 @@ protected:
     void setupActions();
     
     bool queryExit();
+    virtual void readProperties(KConfig* config);
+    virtual void saveProperties(KConfig* config);
 
 private:
     void readSettings();
     void writeSettings();
 
-    KParts::ReadOnlyPart *part;
+    KParts::ReadOnlyPart* m_part;
     QString               m_lastOpenDir;
 };
 
+
 #endif // CERVISIASHELL_H
 
 // Local Variables:
