@@ -3,7 +3,7 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2001-2004 by Stanislav Visnovsky
+		2001-2005 by Stanislav Visnovsky
 			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+  
 **************************************************************************** */
 #include <qtextstream.h>
 #include <qfile.h>
@@ -54,12 +65,7 @@
 
 #include "resources.h"
 #include "version.h"
-
-// from libgettext
-extern "C"
-{
-#include "libgettext/fstrcmp.h"
-}
+#include "stringdistance.h"
 
 using namespace KBabel;
 
@@ -121,7 +127,7 @@ QString Catalog::context(uint index) const
     QStringList lines = QStringList::split("\n",c);
 
     QString result;
-    for( QStringList::Iterator it=lines.begin(); it!=lines.end(); it++)
+    for( QStringList::Iterator it=lines.begin(); it!=lines.end(); ++it)
     {
 	if( (*it).startsWith( "#:") )
 	{
@@ -146,7 +152,7 @@ int Catalog::indexForMsgid(const QString& id) const
     int i=0;
 	QValueVector<CatalogItem>::ConstIterator it = d->_entries.begin();
 
-    while(it != d->_entries.end() && !((*it).msgid().contains(id)))
+    while(it != d->_entries.end() && !((*it).msgid(true).contains(id)))
     {
         ++it;
         i++;
@@ -269,7 +275,16 @@ bool Catalog::setHeader(CatalogItem newHeader)
 {
    if(newHeader.isValid())
    {
+      // normalize the values - ensure every key:value pair is only on a single line
+      QString values = newHeader.msgstr().first();
+      values.replace ("\n", "");
+      values.replace ("\\n", "\\n\n");
+      
+      kdDebug () << "Normalized header: " << values << endl;
+      
       d->_header=newHeader;
+      d->_header.setMsgstr (values);
+      
       setModified(true);
 
       emit signalHeaderChanged();
@@ -299,14 +314,18 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
    QStringList::Iterator it,ait;
    QString temp;
    bool found;
-   if(!usePrefs || d->_saveSettings.updateLastTranslator)
+
+   IdentitySettings identityOptions = identitySettings();
+   SaveSettings saveOptions = saveSettings();
+   
+   if(!usePrefs || saveOptions.updateLastTranslator)
    {
       found=false;
 
-      temp="Last-Translator: "+d->_identitySettings.authorName;
-      if(!d->_identitySettings.authorEmail.isEmpty())
+      temp="Last-Translator: "+identityOptions.authorName;
+      if(!identityOptions.authorEmail.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.authorEmail+">");
+         temp+=(" <"+identityOptions.authorEmail+">");
       }
       temp+="\\n";
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -322,7 +341,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateRevisionDate)
+   if(!usePrefs || saveOptions.updateRevisionDate)
    {
       found=false;
 
@@ -341,11 +360,11 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateProject)
+   if(!usePrefs || saveOptions.updateProject)
    {
       found=false;
 
-      temp="Project-Id-Version: "+d->_saveSettings.projectString+"\\n";
+      temp="Project-Id-Version: "+saveOptions.projectString+"\\n";
       temp.replace( "@PACKAGE@", packageName());
 
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -361,14 +380,14 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateLanguageTeam)
+   if(!usePrefs || saveOptions.updateLanguageTeam)
    {
       found=false;
 
-      temp="Language-Team: "+d->_identitySettings.languageName;
-      if(!d->_identitySettings.mailingList.isEmpty())
+      temp="Language-Team: "+identityOptions.languageName;
+      if(!identityOptions.mailingList.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.mailingList+">");
+         temp+=(" <"+identityOptions.mailingList+">");
       }
       temp+="\\n";
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -384,18 +403,18 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateCharset)
+   if(!usePrefs || saveOptions.updateCharset)
    {
       found=false;
 
 	  QString encodingStr;
-      if(d->_saveSettings.useOldEncoding && d->fileCodec)
+      if(saveOptions.useOldEncoding && d->fileCodec)
       {
 		  encodingStr = charsetString(d->fileCodec);
       }
       else
       {
-          encodingStr=charsetString(d->_saveSettings.encoding);
+          encodingStr=charsetString(saveOptions.encoding);
       }
       temp="Content-Type: text/plain; charset="+encodingStr+"\\n";
       QString charsettemp="; charset="+encodingStr+"\\n";
@@ -413,7 +432,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateEncoding)
+   if(!usePrefs || saveOptions.updateEncoding)
    {
       found=false;
 
@@ -468,11 +487,11 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 
 
    temp="Plural-Forms: %1\\n";
-   temp=temp.arg(d->_identitySettings.gnuPluralFormHeader);
+   temp=temp.arg(identityOptions.gnuPluralFormHeader);
    found=false;
 
    // update plural form header
-   if( !d->_identitySettings.gnuPluralFormHeader.isEmpty() )
+   if( !identityOptions.gnuPluralFormHeader.isEmpty() )
    {
 	for( it = headerList.begin(); it != headerList.end(); ++it )
 	{
@@ -483,7 +502,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
     	    }
 	}
     }
-    if(!found && !d->_identitySettings.gnuPluralFormHeader.isEmpty() )
+    if(!found && !identityOptions.gnuPluralFormHeader.isEmpty() )
     {
        headerList.append(temp);
     }
@@ -499,7 +518,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
    oldHeader.setMsgstr(msgstr);
 
    //comment = description, copyrights
-   if(!usePrefs || (d->_saveSettings.FSFCopyright != NoChange))
+   if(!usePrefs || (saveOptions.FSFCopyright != ProjectSettingsBase::NoChange))
    {
       found=false;
 
@@ -516,10 +535,12 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 	    if((*it).contains(QRegExp("^# *Copyright \\(C\\) YEAR Free Software Foundation, Inc\\.")))
 	    {
 		//template string
-    		if(d->_saveSettings.FSFCopyright == Remove) (*it).replace(" YEAR Free Software Foundation, Inc","");
-		else (*it).replace("YEAR", QDate::currentDate().toString("yyyy"));
+    		if( saveOptions.FSFCopyright == ProjectSettingsBase::Remove) 
+		    (*it).replace(" YEAR Free Software Foundation, Inc","");
+		else 
+		    (*it).replace("YEAR", QDate::currentDate().toString("yyyy"));
 	    } else
-	    if( d->_saveSettings.FSFCopyright == Update )
+	    if( saveOptions.FSFCopyright == ProjectSettingsBase::Update )
 	    {
 		    //update years
 		    QString cy = QDate::currentDate().toString("yyyy");
@@ -538,19 +559,26 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 	}
    }
 
-   if(!usePrefs || d->_saveSettings.updateDescription)
+   if(!usePrefs || saveOptions.updateDescription)
    {
-      temp="# "+d->_saveSettings.descriptionString;
+      temp="# "+saveOptions.descriptionString;
       temp.replace( "@PACKAGE@", packageName());
-      temp.replace( "@LANGUAGE@", d->_identitySettings.languageName);
+      temp.replace( "@LANGUAGE@", identityOptions.languageName);
+      // strip the ending whitespace
+      temp.replace( QRegExp("\\s*$"), "" );
+      
+      kdDebug () << "Temp is '" << temp << "'" << endl;
 
       found=false;
       bool foundTemplate=false;
 
       for( it = commentList.begin(); it != commentList.end(); ++it )
       {
+         kdDebug () << "testing '" << (*it) << "'" << endl;
+
          if((*it).contains(QRegExp("^"+temp+"$")))
          {
+	    kdDebug () << "Match " << endl;
             found=true;
          }
 	 if((*it).contains(QRegExp("^# SOME DESCRIPTIVE TITLE.$")))
@@ -563,14 +591,14 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
        if(!found) commentList.prepend(temp);
    }
 
-   if(!usePrefs || d->_saveSettings.updateTranslatorCopyright)
+   if(!usePrefs || saveOptions.updateTranslatorCopyright)
    {
       QStringList foundAuthors;
 
-      temp="# "+d->_identitySettings.authorName;
-      if(!d->_identitySettings.authorEmail.isEmpty())
+      temp="# "+identityOptions.authorName;
+      if(!identityOptions.authorEmail.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.authorEmail+">");
+         temp+=(" <"+identityOptions.authorEmail+">");
       }
       temp+=", "+QDate::currentDate().toString("yyyy")+".";
 
@@ -597,8 +625,8 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 
 	  for( it = foundAuthors.begin() ; it!=foundAuthors.end(); ++it )
 	  {
-		if( (*it).contains(QRegExp(d->_identitySettings.authorName+".*"
-		    +d->_identitySettings.authorEmail)))
+		if( (*it).contains(QRegExp(identityOptions.authorName+".*"
+		    +identityOptions.authorEmail)))
 		{
 		    foundAuthor = true;
 		    if( (*it).contains(QRegExp(cy))) found = true;
@@ -706,7 +734,7 @@ QString Catalog::packageName() const
 
     QString package=d->_url.fileName();
 
-    int index=package.find(QRegExp("(\\."+d->_identitySettings.languageCode+")?\\.pot?$"));
+    int index=package.find(QRegExp("(\\."+identitySettings().languageCode+")?\\.pot?$"));
 
     if(index>0)
       package=package.left(index);
@@ -743,14 +771,16 @@ QString Catalog::packageDir() const
 
 QString Catalog::encoding() const
 {
+    SaveSettings options = saveSettings();
+    
     QString encodingStr;
-    if(d->_saveSettings.useOldEncoding && d->fileCodec)
+    if(options.useOldEncoding && d->fileCodec)
     {
 	    encodingStr = charsetString(d->fileCodec);
     }
     else
     {
-        encodingStr= charsetString(d->_saveSettings.encoding);
+        encodingStr= charsetString(options.encoding);
     }
 
     return encodingStr;
@@ -807,7 +837,7 @@ ConversionStatus Catalog::openURL(const KURL& url, const QString& package)
 
 	connect( this, SIGNAL( signalStopActivity() ),
 	    filter, SLOT( stop() ));
-
+	    
 	// load in the file (target is always local)
 	d->_active = true;
 	kdDebug(KBABEL) << "openURL active" << endl;
@@ -1184,6 +1214,7 @@ void Catalog::modifyUsingTool(KDataTool* tool, const QString& command)
 
 void Catalog::clear()
 {
+    d->_errorIndex.clear();
     d->_entries.clear();
     d->_url=KURL();
     d->_obsoleteEntries.clear();
@@ -1432,10 +1463,10 @@ void Catalog::useProject(Project::Ptr project)
     d->_project->config()->sync();
     d->_project = project;
     readPreferences();
-    emit signalSettingsChanged(d->_saveSettings);
-    emit signalSettingsChanged(d->_identitySettings);
-    emit signalSettingsChanged(d->_miscSettings);
-    emit signalSettingsChanged(d->_tagSettings);
+    emit signalSettingsChanged(saveSettings());
+    emit signalSettingsChanged(identitySettings());
+    emit signalSettingsChanged(miscSettings());
+    emit signalSettingsChanged(tagSettings());
 }
 
 Project::Ptr Catalog::project() const
@@ -1445,13 +1476,8 @@ Project::Ptr Catalog::project() const
 
 void Catalog::readPreferences()
 {
-    d->_saveSettings = d->_project->saveSettings();
-    d->_identitySettings = d->_project->identitySettings();
-
     getNumberOfPluralForms();
     
-    d->_miscSettings = d->_project->miscSettings();
-
     d->_project->config()->setGroup("Tags");
     d->_tagSettings.tagExpressions=d->_project->config()->readListEntry("TagExpressions");
     if( d->_tagSettings.tagExpressions.empty() ) d->_tagSettings.tagExpressions = Defaults::Tag::tagExpressions();
@@ -1464,12 +1490,6 @@ void Catalog::readPreferences()
 
 void Catalog::savePreferences()
 {
-   KConfigGroupSaver groupSaver(d->_project->config(),"Header");
-
-   d->_project->setSettings(d->_saveSettings);
-   d->_project->setSettings(d->_identitySettings);
-   d->_project->setSettings(d->_miscSettings);
-
    d->_project->config()->setGroup("Tags");
 
    d->_project->config()->writeEntry( "TagExpressions", d->_tagSettings.tagExpressions );
@@ -1480,19 +1500,18 @@ void Catalog::savePreferences()
 
 IdentitySettings Catalog::identitySettings() const
 {
-    return d->_identitySettings;
-
+    return d->_project->identitySettings();
 }
 
 SaveSettings Catalog::saveSettings() const
 {
-    return d->_saveSettings;
+    return d->_project->saveSettings();
 
 }
 
 MiscSettings Catalog::miscSettings() const
 {
-    return d->_miscSettings;
+    return d->_project->miscSettings();
 
 }
 
@@ -1519,25 +1538,25 @@ bool Catalog::isReadOnly() const
 
 void Catalog::setSettings(SaveSettings settings)
 {
-   d->_saveSettings=settings;
-
-   emit signalSettingsChanged(settings);
+   d->_project->setSettings(settings);
 }
 
 void Catalog::setSettings(IdentitySettings settings)
 {
-   QString oldLanguageCode = d->_identitySettings.languageCode;
-   int oldForms =d->_identitySettings.numberOfPluralForms;
+   IdentitySettings oldsettings = d->_project->identitySettings();
+   
+   QString oldLanguageCode = oldsettings.languageCode;
+   int oldForms = oldsettings.numberOfPluralForms;
 
 
-   d->_identitySettings=settings;
+   d->_project->setSettings(settings);
 
-   if(oldLanguageCode != d->_identitySettings.languageCode)
+   if(oldLanguageCode != settings.languageCode)
    {
        getNumberOfPluralForms();
    }
 
-    if(oldForms != d->_identitySettings.numberOfPluralForms)
+    if(oldForms != settings.numberOfPluralForms)
     {
         getNumberOfPluralForms();
     }
@@ -1547,9 +1566,8 @@ void Catalog::setSettings(IdentitySettings settings)
 
 void Catalog::setSettings(MiscSettings settings)
 {
-	d->_miscSettings=settings;
-
-	emit signalSettingsChanged(settings);
+    d->_project->setSettings(settings);
+    emit signalSettingsChanged(settings);
 }
 
 void Catalog::setSettings(TagSettings settings)
@@ -1652,18 +1670,22 @@ QString Catalog::dateTime() const
     QString dateTimeString;
     QDate date=QDate::currentDate();
     QTime time=QTime::currentTime();
+    
+    SaveSettings options = d->_project->saveSettings();
 
-    switch(d->_saveSettings.dateFormat)
+    switch(options.dateFormat)
     {
        case Qt::LocalDate:
        {
           return KGlobal::locale()->formatDateTime(QDateTime::currentDateTime());
        }
        case Qt::ISODate:
-          dateTimeString = Defaults::Save::customDateFormat();
+	  d->_project->settings()->useDefaults( true );
+          dateTimeString = d->_project->settings()->customDateFormat();
+	  d->_project->settings()->useDefaults( false );
           break;
        case Qt::TextDate:
-          dateTimeString = d->_saveSettings.customDateFormat;
+          dateTimeString = options.customDateFormat;
           break;
     }
 
@@ -1760,12 +1782,13 @@ QString Catalog::dateTime() const
     }
 
     // timezone
-    dateTimeString.replace( "%Z", d->_identitySettings.timeZone );
+    dateTimeString.replace( "%Z", d->_project->identitySettings().timeZone );
     QTime t;
     int sgn = KRFCDate::localUTCOffset() < 0 ? -1 : 1 ;
     t = t.addSecs( sgn*KRFCDate::localUTCOffset()*60 );
     dateTimeString.replace( "%z", (sgn<0 ? "-" : "+") +t.toString("hhmm"));
 
+    kdDebug () << "New date: " << dateTimeString;
     return dateTimeString;
 }
 
@@ -1797,7 +1820,7 @@ ConversionStatus Catalog::saveFileAs(const KURL &url, bool overwrite)
    }
 
 
-   if(d->_saveSettings.autoUpdate)
+   if(d->_project->saveSettings().autoUpdate)
    {
       d->_header=updatedHeader(d->_header);
       emit signalHeaderChanged();
@@ -1838,8 +1861,7 @@ ConversionStatus Catalog::saveFileAs(const KURL &url, bool overwrite)
 
       if(status == OK)
       {
-         KURL temp(tempFile);
-         if( !KIO::NetAccess::upload( temp.url(), targetURL, NULL ) )
+         if( !KIO::NetAccess::upload( tempFile, targetURL, NULL ) )
          {
             status = OS_ERROR;
          }
@@ -1905,7 +1927,7 @@ ConversionStatus Catalog::writeFile(QString localFile , bool overwrite)
     CatalogExportPlugin* filter=0;
 
     // gimme plugin for this MIME type
-    KMimeType::Ptr mime = KMimeType::findByURL( KURL::fromPathOrURL( localFile ) );
+    KMimeType::Ptr mime = KMimeType::findByURL( KURL::fromPathOrURL( currentURL().url() ) );
     KTrader::OfferList offers = KTrader::self()->query("KBabelFilter", "('"+mime->name()+"' in [X-KDE-Export])");
     KService::Ptr ptr = offers.first();
 
@@ -2339,6 +2361,8 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 	d->_active=true;
 	d->_stop=false;
 	connect( this, SIGNAL( signalStopActivity() ), this, SLOT( stopInternal() ));
+	
+	MiscSettings miscOptions = miscSettings();
 
 	len=0;
 	int pos=0;
@@ -2410,8 +2434,8 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 
         if(findOpts->ignoreContextInfo)
         {
-            contextInfoPos = d->_miscSettings.contextInfo.search(targetStr);
-	    contextInfoLength = d->_miscSettings.contextInfo.matchedLength();
+            contextInfoPos = miscOptions.contextInfo.search(targetStr);
+	    contextInfoLength = miscOptions.contextInfo.matchedLength();
             if(contextInfoPos >= 0)
             {
                 targetStr.remove(contextInfoPos,contextInfoLength);
@@ -2422,9 +2446,9 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
         }
 
         if(findOpts->ignoreAccelMarker
-                && targetStr.contains(d->_miscSettings.accelMarker))
+                && targetStr.contains(miscOptions.accelMarker))
         {
-            accelMarkerPos = targetStr.find(d->_miscSettings.accelMarker);
+            accelMarkerPos = targetStr.find(miscOptions.accelMarker);
             targetStr.remove(accelMarkerPos,1);
 
             if(docPos.offset > (uint)accelMarkerPos)
@@ -2489,6 +2513,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
@@ -2520,6 +2545,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
@@ -2533,6 +2559,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						{
 							docPos.part = Msgid;
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					else if(findOpts->inMsgstr){
@@ -2555,6 +2582,7 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 						else
 						{
 							docPos.item++;
+							docPos.form = 0;
 						}
 					}
 					break;
@@ -2608,6 +2636,8 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
 	d->_active=true;
 	d->_stop=false;
 	connect( this, SIGNAL( signalStopActivity() ), this, SLOT( stopInternal() ));
+	
+	MiscSettings miscOptions = miscSettings();
 
 	len=0;
 	int pos=0;
@@ -2653,8 +2683,8 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
 
         if(findOpts->ignoreContextInfo)
         {
-            contextInfoPos = d->_miscSettings.contextInfo.search(targetStr);
-	    contextInfoLength = d->_miscSettings.contextInfo.matchedLength();
+            contextInfoPos = miscOptions.contextInfo.search(targetStr);
+	    contextInfoLength = miscOptions.contextInfo.matchedLength();
             if(contextInfoPos >= 0)
             {
                 targetStr.remove(contextInfoPos,contextInfoLength);
@@ -2665,9 +2695,9 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
         }
 
         if(findOpts->ignoreAccelMarker
-                && targetStr.contains(d->_miscSettings.accelMarker))
+                && targetStr.contains(miscOptions.accelMarker))
         {
-            accelMarkerPos = targetStr.find(d->_miscSettings.accelMarker);
+            accelMarkerPos = targetStr.find(miscOptions.accelMarker);
             targetStr.remove(accelMarkerPos,1);
 
             if(docPos.offset > (uint)accelMarkerPos)
@@ -2934,7 +2964,7 @@ Catalog::DiffResult Catalog::diff(uint entry, QString *result)
             QStringList::ConstIterator it;
             for(it = list.begin(); it != list.end(); ++it)
             {
-                double weight = fstrcmp( id.utf8(), (*it).utf8() );
+                double weight = LevenshteinDistance()(id, (*it));
                 if(weight > bestWeight)
                 {
                     bestWeight = weight;
@@ -2973,7 +3003,7 @@ Catalog::DiffResult Catalog::diff(uint entry, QString *result)
                 emit signalProgress(percent);
             }
 
-            double weight = fstrcmp( id.utf8(), (*it).utf8() );
+            double weight = LevenshteinDistance()( id, (*it) );
             if(weight > bestWeight)
             {
                 bestWeight = weight;
@@ -3347,13 +3377,15 @@ int Catalog::defaultNumberOfPluralForms() const
 
 void Catalog::getNumberOfPluralForms()
 {
-    if(d->_identitySettings.numberOfPluralForms > 0)
+    IdentitySettings options = identitySettings();
+    
+    if(options.numberOfPluralForms > 0)
     {
-        d->numberOfPluralForms = d->_identitySettings.numberOfPluralForms;
+        d->numberOfPluralForms = options.numberOfPluralForms;
         return;
     }
 
-    QString lang=d->_identitySettings.languageCode;
+    QString lang=options.languageCode;
     if(lang.isEmpty())
     {
         d->numberOfPluralForms=-1;
@@ -3527,4 +3559,35 @@ QString Catalog::mimeTypes() const
     return d->_mimeTypes;
 }
 
+void Catalog::wordCount (uint &total, uint &fuzzy, uint &untranslated) const
+{
+    total = 0;
+    fuzzy = 0;
+    untranslated = 0;
+    
+    QRegExp separator( "[ \n\t]+" );
+    
+    for ( QValueVector<CatalogItem>::Iterator it = d->_entries.begin();
+					it != d->_entries.end(); ++it)
+    {
+	// find out the number of words for this message
+	
+	// join all forms together
+	QString message = (*it).msgid ().join (" ");
+	
+	// remove tags first
+	d->_tagExtractor->setString( message );
+	message = d->_tagExtractor->plainString(false);
+
+	QStringList words = QStringList::split ( separator, message );
+	
+	total += words.count();
+
+	if ( (*it).isFuzzy() )
+	    fuzzy += words.count();
+	else if ( (*it).isUntranslated() )
+	    untranslated += words.count();
+    }    
+}
+
 #include "catalog.moc"
