@@ -18,6 +18,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+	    
 **************************************************************************** */
 
 #ifndef SEARCH_ENGINE_H
@@ -37,7 +48,7 @@
  * Information, that is not available in a specific implementation of
  * the SearchEngine, should be left empty.
  */
-class TranslationInfo
+class KDE_EXPORT TranslationInfo
 {
 public:
     /** 
@@ -96,7 +107,7 @@ public:
  * plus additional information where it was found,
  * the date and time of the last change, the translator, etc.
  */
-class SearchResult
+class KDE_EXPORT SearchResult
 {
 public:
     SearchResult();
@@ -151,7 +162,7 @@ public:
 /**
  *
  */
-class SearchFilter 
+class KDE_EXPORT SearchFilter 
 {
 public:
     SearchFilter() : 
@@ -230,7 +241,7 @@ private:
  * necessary function. The widget should not be bigger than 400x400.
  * If you need more space, you maybe want to use a tabbed widget.
  */
-class PrefWidget : public QWidget
+class KDE_EXPORT PrefWidget : public QWidget
 {
     Q_OBJECT
             
@@ -254,7 +265,7 @@ public slots:
 };
 
 
-class SearchEngine : public QObject
+class KDE_EXPORT SearchEngine : public QObject
 {
     Q_OBJECT
             
