@@ -579,7 +579,8 @@ void kio_svnProtocol::mkdir( const KURL::List& list, int /*permissions*/ ) {
 	KURL::List::const_iterator it = list.begin(), end = list.end();
 	for ( ; it != end; ++it ) {
 		QString cur = makeSvnURL( *it );
-		const char *_target = apr_pstrdup( subpool, cur.utf8() );
+		kdDebug() << "kio_svnProtocol::mkdir raw url for subversion : " << cur << endl;
+		const char *_target = apr_pstrdup( subpool, svn_path_canonicalize( apr_pstrdup( subpool, cur.utf8() ), subpool ) );
 		(*(( const char ** )apr_array_push(( apr_array_header_t* )targets)) ) = _target;
 	}
 
@@ -840,9 +841,12 @@ void kio_svnProtocol::import( const KURL& repos, const KURL& wc ) {
 	nurl.setProtocol( chooseProtocol( repos.protocol() ) );
 	dest.setProtocol( "file" );
 	recordCurrentURL( nurl );
+	dest.cleanPath( true ); // remove doubled '/'
 	QString source = dest.path();
-	QString target;
-	target = makeSvnURL( repos );
+	if ( source.endsWith( "/" ) ) 
+		source = source.left( source.length()-1 );
+	QString target = makeSvnURL( repos );
+ 
 
 	const char *path = svn_path_canonicalize( apr_pstrdup( subpool, source.utf8() ), subpool );
 	const char *url = svn_path_canonicalize( apr_pstrdup( subpool, target.utf8() ), subpool );
@@ -1037,29 +1041,30 @@ void kio_svnProtocol::wc_status(const KURL& wc, bool checkRepos, bool fullRecurs
 QString kio_svnProtocol::makeSvnURL ( const KURL& url ) const {
 	QString kproto = url.protocol();
 	KURL tpURL = url;
+	tpURL.cleanPath( true );
 	QString svnUrl;
 	if ( kproto == "svn+http" ) {
 		kdDebug() << "http:/" << url.url() << endl;
 		tpURL.setProtocol("http");
-		svnUrl = tpURL.url();
+		svnUrl = tpURL.url(-1);
 		return svnUrl;
 	}
 	else if ( kproto == "svn+https" ) {
 		kdDebug() << "https:/" << url.url() << endl;
 		tpURL.setProtocol("https");
-		svnUrl = tpURL.url();
+		svnUrl = tpURL.url(-1);
 		return svnUrl;
 	}
 	else if ( kproto == "svn+ssh" ) {
 		kdDebug() << "svn+ssh:/" << url.url() << endl;
 		tpURL.setProtocol("svn+ssh");
-		svnUrl = tpURL.url();
+		svnUrl = tpURL.url(-1);
 		return svnUrl;
 	}
 	else if ( kproto == "svn" ) {
 		kdDebug() << "svn:/" << url.url() << endl;
 		tpURL.setProtocol("svn");
-		svnUrl = tpURL.url();
+		svnUrl = tpURL.url(-1);
 		return svnUrl;
 	}
 	else if ( kproto == "svn+file" ) {
