@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -11,7 +16,6 @@
 #define UMLCLASS_H
 
 #include "classifier.h"
-#include "umltemplatelist.h"
 
 /**
  * This class contains the non-graphical information required for a UML Class.
@@ -36,7 +40,7 @@ public:
 	 * @param name		The name of the class.
 	 * @param id		The unique id of the class.
 	 */
-	UMLClass(const QString & Name = "", int id = -1);
+	UMLClass(const QString & Name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Copy the internal presentation of this object into the new
@@ -59,6 +63,14 @@ public:
 	 */
 	bool operator==( UMLClass & rhs );
 
+	/**
+	 * Creates an attribute for the class.
+	 *
+	 * @param name  An optional name, used by when creating through UMLListView
+	 * @return	The UMLAttribute created
+	 */
+	UMLObject* createAttribute(const QString &name = QString::null);
+
 	/**
 	 * Adds an attribute to the class.
 	 * If an attribute of the given name already exists, then
@@ -71,7 +83,9 @@ public:
 	 *                      will internally assign a new ID.
 	 * @return	Pointer to the UMLAttribute created or found.
 	 */
-	UMLAttribute* addAttribute(QString name, int id = -1);
+	UMLAttribute* addAttribute(const QString &name, Uml::IDType id = Uml::id_None);
+
+	UMLAttribute* addAttribute(const QString &name, UMLObject *type, Uml::Scope scope);
 
 	/**
 	 * Adds an already created attribute.
@@ -103,60 +117,11 @@ public:
 	 * the returned attribute (or to delete the attribute)
 	 *
 	 * @param  a		attribute to take
+	 * @param wasAtIndex	if given, the index in m_List of the item taken
+	 *			is returned in the int pointed-to.
 	 * @return	pointer to the attribute or null if not found.
 	 */
-	UMLAttribute* takeAttribute(UMLAttribute* a);
-
-	/**
-	 * Adds a template to the class.
-	 *
-	 * @param name		The name of the template.
-	 * @param id		The id of the template.
-	 * @return	Pointer to the UMLTemplate object created.
-	 */
-	UMLObject* addTemplate(QString name, int id);
-
-	/**
-	 * Adds an already created template.
-	 * The template object must not belong to any other concept.
-	 *
-	 * @param newTemplate	Pointer to the UMLTemplate object to add.
-	 * @param log		Pointer to the IDChangeLog.
-	 * @return	True if the template was sucessfully added.
-	 */
-	bool addTemplate(UMLTemplate* newTemplate, IDChangeLog* log = 0);
-
-	/**
-	 * Adds an template to the class.
-	 * The template object must not belong to any other class.
-	 *
-	 * @param Template	Pointer to the UMLTemplate to add.
-	 * @param position	The position of the template in the list.
-	 *			A value of -1 will add the template at the end.
-	 * @return	True if the template was sucessfully added.
-	 */
-	 //TODO: if the param IDChangeLog from the method above is not being used,
-	  // give position a default value of -1 and the method can replace the above one
-	bool addTemplate(UMLTemplate* Template, int position);
-
-	/**
-	 * Removes a template from the class.
-	 *
-	 * @param template	The template to remove.
-	 * @return	Count of the remaining templates after removal.
-	 *		Returns -1 if the given template was not found.
-	 */
-	int removeTemplate(UMLTemplate* umltemplate);
-
-	/**
-	 * Take and return a templatee from class.
-	 * It is the callers responsibility to pass on ownership of
-	 * the returned template (or to delete the template)
-	 *
-	 * @param t		template to take
-	 * @return	pointer to the template or null if not found.
-	 */
-	UMLTemplate* takeTemplate(UMLTemplate* t);
+	UMLAttribute* takeAttribute(UMLAttribute* a, int *wasAtIndex = NULL);
 
 	/**
 	 * Returns the number of attributes for the class.
@@ -166,71 +131,13 @@ public:
 	int attributes();
 
 	/**
-	 * Returns the number of templates for the class.
+	 * Returns the attributes.
+	 * Same as UMLClassifier::getFilteredList(ot_Attribute) but
+	 * return type is a true UMLAttributeList.
 	 *
-	 * @return	The number of templates for the class.
+	 * @return	List of true attributes for the class.
 	 */
-	int templates();
-
-	/**
-	 * Return the list of attributes for the class.
-	 *
-	 * @return	Pointer to the list of attributes for the class.
-	 */
-	UMLClassifierListItemList* getAttList();
-
-	/**
-	 * Returns the entries in m_pAttList that are actually attributes.
-	 *
-	 * @return	Pointer to the list of true attributes for the class.
-	 */
-	UMLAttributeList* getFilteredAttributeList();
-
-	/**
-	 * Return the list of templates for the class.
-	 *
-	 * @return	Pointer to the list of templates for the class.
-	 */
-	UMLClassifierListItemList* getTemplateList();
-
-	/**
-	 * Returns the entries in m_pTemplatesList that are actually templates
-	 *
-	 * @return	Pointer to the list of true templates for the class.
-	 */
-	UMLTemplateList* getFilteredTemplateList();
-
-	/**
-	 * Find a list of attributes, operations, associations or
-	 * templates with the given name.
-	 *
-	 * @param t		The type to find.
-	 * @param n		The name of the object to find.
-	 * @param seekStereo	Set this true if we should look at the object's
-	 *			stereotype instead of its name.
-	 * @return	List of objects found.  Will be empty if none found.
-	 */
-	 virtual UMLObjectList findChildObject(UMLObject_Type t, QString n,
-					       bool seekStereo = false);
-
-	/**
-	 * Find an attribute, operation, association or template.
-	 *
-	 * @param id		The id of the object to find.
-	 * @return	Pointer to the object found; NULL if not found.
-	 */
-	virtual UMLObject* findChildObject(int id);
-
-	/**
-	 * Find the object of the given (non-numeric) auxiliary ID in
-	 * the lists of contained objects.  The auxiliary ID is the ID
-	 * returned by UMLObject::getAuxId() and is currently only used
-	 * for loading foreign XMI files.
-	 *
-	 * @param idStr		The ID to seek.
-	 * @return	Pointer to the UMLObject found or NULL if not found.
-	 */
-	UMLObject* findChildObjectByIdStr(QString idStr);
+	UMLAttributeList getAttributeList();
 
 	/**
 	 * Return true if this class has an enumeration stereotype.
@@ -260,8 +167,6 @@ public:
 signals:
 	void attributeAdded(UMLObject*);
 	void attributeRemoved(UMLObject*);
-	void templateAdded(UMLObject*);
-	void templateRemoved(UMLObject*);
 
 protected:
 	/**
@@ -277,15 +182,6 @@ private:
 	 */
 	void init();
 
-	/**
-	 * List of all the attributes in this class.
-	 */
-	UMLClassifierListItemList m_AttsList;
-
-	/**
-	 * List of all the templates in this class.
-	 */
-	UMLClassifierListItemList m_TemplateList;
 };
 
 #endif // UMLCLASS_H
