@@ -12,18 +12,22 @@
 #include "../umldoc.h"
 #include "../class.h"
 #include "../enum.h"
+#include "../entity.h"
 #include "../attribute.h"
 #include "../operation.h"
 #include "../template.h"
 #include "../enumliteral.h"
+#include "../entityattribute.h"
 #include <kbuttonbox.h>
 #include <kdebug.h>
 #include <klocale.h>
 #include <qlayout.h>
 
+using namespace Uml;
+
 ClassifierListPage::ClassifierListPage(QWidget* parent, UMLClassifier* classifier,
-				       UMLDoc* doc, UMLObject_Type type) : QWidget(parent) {
-	itemType = type;
+				       UMLDoc* doc, Object_Type type) : QWidget(parent) {
+	m_itemType = type;
 	QString typeName("");
 	QString newItemType("");
 	if (type == ot_Attribute) {
@@ -38,6 +42,9 @@ ClassifierListPage::ClassifierListPage(QWidget* parent, UMLClassifier* classifie
 	} else if (type == ot_EnumLiteral) {
 		typeName = i18n("Enum Literals");
 		newItemType = i18n("N&ew Enum Literal...");
+	} else if (type == ot_EntityAttribute) {
+		typeName = i18n("Entity Attributes");
+		newItemType = i18n("N&ew Entity Attribute...");
 	} else {
 		kdWarning() << "unknown listItem type in ClassifierListPage" << endl;
 	}
@@ -93,11 +100,11 @@ ClassifierListPage::ClassifierListPage(QWidget* parent, UMLClassifier* classifie
 	mainLayout->addWidget(m_pDocGB);
 
 	UMLClassifierListItemList itemList(getItemList());
-	
+
 	// add each item in the list to the ListBox and connect each item modified signal
 	// to the ListItemModified slot in this class
 	for (UMLClassifierListItem* listItem = itemList.first(); listItem != 0; listItem = itemList.next() ) {
-		m_pItemListLB->insertItem(listItem->getShortName());
+		m_pItemListLB->insertItem(listItem->toString(Uml::st_SigNoScope));
 		connect( listItem, SIGNAL(modified()),this,SLOT(slotListItemModified()) );
 	}
 
@@ -184,17 +191,19 @@ void ClassifierListPage::slotClicked(QListBoxItem*item) {
 		listItem = getItemList().at( m_pItemListLB->index(item) );
 	}
 
-	//now update screen
-	m_pDocTE->setText( listItem->getDoc() );
-	enableWidgets(true);
-	m_pOldListItem = listItem;
+	if (listItem) {
+		//now update screen
+		m_pDocTE->setText( listItem->getDoc() );
+		enableWidgets(true);
+		m_pOldListItem = listItem;
+	}
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 void ClassifierListPage::updateObject() {
 	saveCurrentItemDocumentation();
 	QListBoxItem*i = m_pItemListLB->item(m_pItemListLB->currentItem());
 	slotClicked(i);
-	
+
 	// The rest of this function does nothing?!
 	QStringList stringList;
 	int count = m_pItemListLB->count();
@@ -210,7 +219,7 @@ void ClassifierListPage::slotListItemCreated(UMLObject* object) {
 		return;
 	}
 	int index = m_pItemListLB->count();
-	m_pItemListLB ->insertItem((static_cast<UMLClassifierListItem*>(object))->getShortName(), index);
+	m_pItemListLB->insertItem(object->getName(), index);
 	m_bSigWaiting = false;
 
 	// now select the new item, so that the user can go on adding doc or calling
@@ -226,7 +235,7 @@ void ClassifierListPage::slotListItemModified() {
 	//is this safe???
 	UMLClassifierListItem* object = const_cast<UMLClassifierListItem*>(dynamic_cast<const UMLClassifierListItem*>(sender()));
 	int index = m_pItemListLB->currentItem();
-	m_pItemListLB->changeItem(object->getShortName(), index);
+	m_pItemListLB->changeItem(object->getName(), index);
 	m_bSigWaiting = false;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -242,26 +251,30 @@ void ClassifierListPage::slotRightButtonClicked(QListBoxItem* /*item*/, const QP
 void ClassifierListPage::slotRightButtonPressed(QListBoxItem* item, const QPoint& p) {
 	ListPopupMenu::Menu_Type type = ListPopupMenu::mt_Undefined;
 	if (item) { //pressed on a list item
-		if (itemType == ot_Attribute) {
+		if (m_itemType == ot_Attribute) {
 			type = ListPopupMenu::mt_Attribute_Selected;
-		} else if (itemType == ot_Operation) {
+		} else if (m_itemType == ot_Operation) {
 			type = ListPopupMenu::mt_Operation_Selected;
-		} else if (itemType == ot_Template) {
+		} else if (m_itemType == ot_Template) {
 			type = ListPopupMenu::mt_Template_Selected;
-		} else if (itemType == ot_EnumLiteral) {
+		} else if (m_itemType == ot_EnumLiteral) {
 			type = ListPopupMenu::mt_EnumLiteral_Selected;
+		} else if (m_itemType == ot_EntityAttribute) {
+			type = ListPopupMenu::mt_EntityAttribute_Selected;
 		} else {
 			kdWarning() << "unknown type in ClassifierListPage" << endl;
 		}
 	} else { //pressed into fresh air
-		if (itemType == ot_Attribute) {
+		if (m_itemType == ot_Attribute) {
 			type = ListPopupMenu::mt_New_Attribute;
-		} else if (itemType == ot_Operation) {
+		} else if (m_itemType == ot_Operation) {
 			type = ListPopupMenu::mt_New_Operation;
-		} else if (itemType == ot_Template) {
+		} else if (m_itemType == ot_Template) {
 			type = ListPopupMenu::mt_New_Template;
-		} else if (itemType == ot_EnumLiteral) {
+		} else if (m_itemType == ot_EnumLiteral) {
 			type = ListPopupMenu::mt_New_EnumLiteral;
+		} else if (m_itemType == ot_EntityAttribute) {
+			type = ListPopupMenu::mt_New_EntityAttribute;
 		} else {
 			kdWarning() << "unknown type in ClassifierListPage" << endl;
 		}
@@ -289,6 +302,7 @@ void ClassifierListPage::slotPopupMenuSel(int id) {
 		case ListPopupMenu::mt_New_Operation:
 		case ListPopupMenu::mt_New_Template:
 		case ListPopupMenu::mt_New_EnumLiteral:
+		case ListPopupMenu::mt_New_EntityAttribute:
 			slotNewListItem();
 			break;
 
@@ -322,10 +336,14 @@ void ClassifierListPage::slotUpClicked() {
 	//set the moved item selected
 	QListBoxItem* item = m_pItemListLB->item( index - 1 );
 	m_pItemListLB->setSelected( item, true );
-	
+
 	//now change around in the list
 	UMLClassifierListItem* currentAtt = getItemList().at( index );
-	takeClassifier(currentAtt);
+	// NB: The index in the m_pItemListLB is not necessarily the same
+	//     as the index in the UMLClassifier::m_List.
+	//     Reason: getItemList() returns only a subset of all entries
+	//     in UMLClassifier::m_List.
+	takeClassifier(currentAtt, index);  // now we index the UMLClassifier::m_List
 	addClassifier(currentAtt, index - 1);
 	slotClicked( item );
 }
@@ -347,8 +365,12 @@ void ClassifierListPage::slotDownClicked() {
 	m_pItemListLB->setSelected( item, true );
 	//now change around in the list
 	UMLClassifierListItem* currentAtt = getItemList().at( index );
-	takeClassifier(currentAtt);
-	addClassifier(currentAtt, index + 2);
+	// NB: The index in the m_pItemListLB is not necessarily the same
+	//     as the index in the UMLClassifier::m_List.
+	//     Reason: getItemList() returns only a subset of all entries
+	//     in UMLClassifier::m_List.
+	takeClassifier(currentAtt, index);  // now we index the UMLClassifier::m_List
+	addClassifier(currentAtt, index + 1);
 	slotClicked( item );
 }
 
@@ -363,7 +385,7 @@ void ClassifierListPage::slotDoubleClick( QListBoxItem* item ) {
 	}
 
 	if( listItem->showPropertiesDialogue(this) ) {
-		m_pItemListLB->changeItem( listItem->getShortName(), m_pItemListLB->index(item) );
+		m_pItemListLB->changeItem( listItem->getName(), m_pItemListLB->index(item) );
 	}
 }
 
@@ -385,7 +407,7 @@ void ClassifierListPage::slotProperties() {
 void ClassifierListPage::slotNewListItem() {
 	saveCurrentItemDocumentation();
 	m_bSigWaiting = true;
-	m_pDoc->createChildObject(m_pClassifier, itemType);
+	m_pDoc->createChildObject(m_pClassifier, m_itemType);
 }
 
 void ClassifierListPage::saveCurrentItemDocumentation() {
@@ -396,62 +418,38 @@ void ClassifierListPage::saveCurrentItemDocumentation() {
 }
 
 UMLClassifierListItemList ClassifierListPage::getItemList() {
-	switch (itemType) {
-		case ot_Attribute: {
-			UMLClass* classifier = dynamic_cast<UMLClass*>(m_pClassifier);
-			if (classifier) {
-				return UMLClassifierListItemList(*classifier->getAttList());
-			}
-			break;
-		}
-		case ot_Operation: {
-			return m_pClassifier->getOpList();
-		}
-		case ot_Template: {
-			UMLClass* classifier = dynamic_cast<UMLClass*>(m_pClassifier);
-			if (classifier) {
-				return UMLClassifierListItemList(*classifier->getTemplateList());
-			}
-			break;
-		}
-		case ot_EnumLiteral: {
-			UMLEnum* classifier = dynamic_cast<UMLEnum*>(m_pClassifier);
-			if (classifier) {
-				return UMLClassifierListItemList(*classifier->getEnumLiteralList());
-			}
-			break;
-		}
-		default: {
-			kdWarning() << "unknown type in ClassifierListPage" << endl;
-			return UMLClassifierListItemList();
-		}
-	}
-	kdError() << "ClassifierListPage is in an inconsistent state!" << endl;
-	return UMLClassifierListItemList();
+	return m_pClassifier->getFilteredList(m_itemType);
 }
-	
-bool ClassifierListPage::addClassifier(UMLClassifierListItem* classifier, int position) {
-	switch (itemType) {
+
+bool ClassifierListPage::addClassifier(UMLClassifierListItem* listitem, int position) {
+	switch (m_itemType) {
 		case ot_Attribute: {
 			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
 			if (c) {
-				return c->addAttribute(dynamic_cast<UMLAttribute*>(classifier), 0, position);
+				return c->addAttribute(dynamic_cast<UMLAttribute*>(listitem), NULL, position);
 			}
 			break;
 		}
 		case ot_Operation: {
-			return m_pClassifier->addOperation(dynamic_cast<UMLOperation*>(classifier), position);
+			return m_pClassifier->addOperation(dynamic_cast<UMLOperation*>(listitem), position);
 		case ot_Template:
 			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
 			if (c) {
-				return c->addTemplate(dynamic_cast<UMLTemplate*>(classifier), position);
+				return c->addTemplate(dynamic_cast<UMLTemplate*>(listitem), position);
 			}
 			break;
 		}
 		case ot_EnumLiteral: {
 			UMLEnum* c = dynamic_cast<UMLEnum*>(m_pClassifier);
 			if (c) {
-				return c->addEnumLiteral(dynamic_cast<UMLEnumLiteral*>(classifier), position);
+				return c->addEnumLiteral(dynamic_cast<UMLEnumLiteral*>(listitem), position);
+			}
+			break;
+		}
+		case ot_EntityAttribute: {
+			UMLEntity* c = dynamic_cast<UMLEntity*>(m_pClassifier);
+			if (c) {
+				return c->addEntityAttribute(dynamic_cast<UMLEntityAttribute*>(listitem), position);
 			}
 			break;
 		}
@@ -460,33 +458,41 @@ bool ClassifierListPage::addClassifier(UMLClassifierListItem* classifier, int po
 			return false;
 		}
 	}
-	kdError() << "ClassifierListPage::addClassifier unable to handle classifier type in current state" << endl;
+	kdError() << "ClassifierListPage::addClassifier unable to handle listitem type in current state" << endl;
 	return false;
 }
 
-UMLClassifierListItem* ClassifierListPage::takeClassifier(UMLClassifierListItem* classifier) {
-	switch (itemType) {
+UMLClassifierListItem* ClassifierListPage::takeClassifier(UMLClassifierListItem* listitem,
+							  int &wasAtIndex) {
+	switch (m_itemType) {
 		case ot_Attribute: {
 			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
 			if (c) {
-				return c->takeAttribute(dynamic_cast<UMLAttribute*>(classifier));
+				return c->takeAttribute(dynamic_cast<UMLAttribute*>(listitem), &wasAtIndex);
 			}
 			break;
 		}
 		case ot_Operation: {
-			return m_pClassifier->takeOperation(dynamic_cast<UMLOperation*>(classifier));
+			return m_pClassifier->takeOperation(dynamic_cast<UMLOperation*>(listitem), &wasAtIndex);
 		}
 		case ot_Template: {
 			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
 			if (c) {
-				return c->takeTemplate(dynamic_cast<UMLTemplate*>(classifier));
+				return c->takeTemplate(dynamic_cast<UMLTemplate*>(listitem), &wasAtIndex);
 			}
 			break;
 		}
 		case ot_EnumLiteral: {
 			UMLEnum* c = dynamic_cast<UMLEnum*>(m_pClassifier);
 			if (c) {
-				return c->takeEnumLiteral(dynamic_cast<UMLEnumLiteral*>(classifier));
+				return c->takeEnumLiteral(dynamic_cast<UMLEnumLiteral*>(listitem), &wasAtIndex);
+			}
+			break;
+		}
+		case ot_EntityAttribute: {
+			UMLEntity* c = dynamic_cast<UMLEntity*>(m_pClassifier);
+			if (c) {
+				return c->takeEntityAttribute(dynamic_cast<UMLEntityAttribute*>(listitem), &wasAtIndex);
 			}
 			break;
 		}
@@ -495,42 +501,10 @@ UMLClassifierListItem* ClassifierListPage::takeClassifier(UMLClassifierListItem*
 			return 0;
 		}
 	}
-	kdError() << "ClassifierListPage::takeClassifier unable to handle classifier type in current state" << endl;
+	kdError() << "ClassifierListPage::takeClassifier unable to handle listitem type in current state" << endl;
 	return 0;
 }
 
-int ClassifierListPage::removeClassifier(UMLClassifierListItem* classifier) {
-	switch (itemType) {
-		case ot_Attribute: {
-			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
-			if (c) {
-				return c->removeAttribute(dynamic_cast<UMLAttribute*>(classifier));
-			}
-			break;
-		}
-		case ot_Operation: {
-			return m_pClassifier->removeOperation(dynamic_cast<UMLOperation*>(classifier));
-		}
-		case ot_Template: {
-			UMLClass* c = dynamic_cast<UMLClass*>(m_pClassifier);
-			if (c) {
-				return c->removeTemplate(dynamic_cast<UMLTemplate*>(classifier));
-			}
-			break;
-		}
-		case ot_EnumLiteral: {
-			UMLEnum* c = dynamic_cast<UMLEnum*>(m_pClassifier);
-			if (c) {
-				return c->removeEnumLiteral(dynamic_cast<UMLEnumLiteral*>(classifier));
-			}
-			break;
-		}
-		default: {
-			kdWarning() << "unknown type in ClassifierListPage" << endl;
-			return -1;
-		}
-	}
-	kdError() << "ClassifierListPage::removeClassifier unable to handle classifier type in current state" << endl;
-	return -1;
-}
+
 #include "classifierlistpage.moc"
+
